@@ -45,6 +45,7 @@ const ClientInterface = '<node> \
     <property name="DesktopId" type="s" access="readwrite"/> \
     <property name="RequestedAccuracyLevel" type="u" access="readwrite"/> \
     <property name="DistanceThreshold" type="u" access="readwrite"/> \
+    <property name="Active" type="b" access="read"/> \
     <method name="Start"/> \
     <method name="Stop"/> \
     <signal name="LocationUpdated"> \
@@ -83,61 +84,24 @@ const Geoclue = new Lang.Class({
         'connected': GObject.ParamSpec.boolean('connected',
                                                'Connected',
                                                'Connected to DBus service',
-                                               GObject.ParamFlags.READABLE,
+                                               GObject.ParamFlags.READABLE |
+                                               GObject.ParamFlags.WRITABLE,
                                                false)
     },
 
+    set connected(c) {
+        this._connected = c;
+        this.notify('connected');
+    },
+
     get connected() {
         return this._connected;
     },
 
-    overrideLocation: function(location) {
-        if (this._clientProxy && this._locationUpdatedId > 0) {
-            this._clientProxy.disconnectSignal(this._locationUpdatedId);
-            this._locationUpdatedId = 0;
-            this._clientProxy.StopRemote(function(result, e) {
-                if (e) {
-                    log ("Failed to connect to GeoClue2 service: " + e.message);
-                }
-            });
-        }
-
-        this._updateLocation(location, true);
-    },
-
-    findLocation: function() {
-        if (!this._clientProxy)
-            return;
-
-        this._locationUpdatedId =
-            this._clientProxy.connectSignal("LocationUpdated",
-                                            this._onLocationUpdated.bind(this));
-
-        this._clientProxy.StartRemote(function(result, e) {
-            if (e) {
-                log ("Failed to connect to GeoClue2 service: " + e.message);
-            }
-        });
-    },
-
     _init: function() {
         this.parent();
-        this._connected = false;
-
-        let lastLocation = Application.settings.get('last-location');
-        if (lastLocation.length >= 3) {
-            let [lat, lng, accuracy] = lastLocation;
-            let lastLocationDescription = Application.settings.get('last-location-description');
-
-            let location = new Geocode.Location({ latitude: lat,
-                                                  longitude: lng,
-                                                  accuracy: accuracy });
-
-            this.place = new Geocode.Place({ location: location,
-                                             name: _("Current location") });
-
-            this.userSetLocation = Application.settings.get('last-location-user-set');
-        }
+        this.connected = false;
+        this.place = null;
 
         try {
             this._managerProxy = new ManagerProxy(Gio.DBus.system,
@@ -164,11 +128,13 @@ const Geoclue = new Lang.Class({
         this._clientProxy.DesktopId = "org.gnome.Maps";
         this._clientProxy.RequestedAccuracyLevel = AccuracyLevel.EXACT;
 
-        if (!this.userSetLocation)
-            this.findLocation();
-
-        this._connected = true;
-        this.notify('connected');
+        this._clientProxy.connectSignal('LocationUpdated',
+                                        this._onLocationUpdated.bind(this));
+        this._clientProxy.StartRemote((function(result, e) {
+            if (e) {
+                log ("Failed to connect to GeoClue2 service: " + e.message);
+            }
+        }).bind(this));
     },
 
     _onLocationUpdated: function(proxy, sender, [oldPath, newPath]) {
@@ -179,20 +145,20 @@ const Geoclue = new Lang.Class({
                                               longitude: geoclueLocation.Longitude,
                                               accuracy: geoclueLocation.Accuracy,
                                               description: geoclueLocation.Description });
-        this._updateLocation(location, false);
+
+        this._updateLocation(location);
+
+        this.connected = this._clientProxy.Active;
+        this._clientProxy.connect('g-properties-changed', (function() {
+            this.connected = this._clientProxy.Active;
+        }).bind(this));
     },
 
-    _updateLocation: function(location, userSet) {
+    _updateLocation: function(location) {
+        if (!this.place)
+            this.place = new Geocode.Place({ name: _("Current location") });
+
         this.place.location = location;
-
-        Application.settings.set('last-location', [location.latitude,
-                                                   location.longitude,
-                                                   location.accuracy]);
-        if (location.description !== null)
-            Application.settings.set('last-location-description', location.description);
-        Application.settings.set('last-location-user-set', userSet);
-        this.userSetLocation = userSet;
-
         this.emit('location-changed');
         Utils.debug("Updated location: " + location.description);
     }
