@@ -22,10 +22,8 @@
 
 const Cairo = imports.gi.cairo;
 const Champlain = imports.gi.Champlain;
-const Geocode = imports.gi.GeocodeGlib;
 const GObject = imports.gi.GObject;
 const Gtk = imports.gi.Gtk;
-
 const Lang = imports.lang;
 const Mainloop = imports.mainloop;
 
@@ -177,7 +175,7 @@ const MapMarker = new Lang.Class({
         this._mapView.onSetMarkerSelected(this);
 
         let markerSelectedSignalId = this._mapView.connect('marker-selected', (function(mapView, selectedMarker) {
-            if (this.get_parent() != selectedMarker.get_parent())
+            if (this.get_parent() !== selectedMarker.get_parent())
                 this.selected = false;
         }).bind(this));
 
