@@ -20,9 +20,6 @@
  * Author: Dario Di Nucci <linkin88mail@gmail.com>
  */
 
-const Geocode = imports.gi.GeocodeGlib;
-const Gtk = imports.gi.Gtk;
-
 const Lang = imports.lang;
 
 const MapBubble = imports.mapBubble;
@@ -38,13 +35,7 @@ const TurnPointBubble = new Lang.Class({
 
         this.parent(params);
 
-        let ui = Utils.getUIObject('turn-point-bubble', [ 'grid',
-                                                          'box-right',
-                                                          'image',
-                                                          'label-title' ]);
-        ui.image.icon_name = turnPoint.iconName;
-        ui.labelTitle.label = turnPoint.instruction;
-
-        this.add(ui.grid);
+        this.image.icon_name = turnPoint.iconName;
+        this.content.hide();
     }
 });
