@@ -21,25 +21,31 @@
  *         Zeeshan Ali (Khattak) <zeeshanak@gnome.org>
  */
 
-const Lang = imports.lang;
-const Gdk = imports.gi.Gdk;
+const Clutter = imports.gi.Clutter;
+const Cogl = imports.gi.Cogl;
 const GLib = imports.gi.GLib;
+const Gdk = imports.gi.Gdk;
+const GdkPixbuf = imports.gi.GdkPixbuf;
+const Geocode = imports.gi.GeocodeGlib;
 const Gio = imports.gi.Gio;
 const Gtk = imports.gi.Gtk;
 const Signals = imports.signals;
-const Geocode = imports.gi.GeocodeGlib;
-const GdkPixbuf = imports.gi.GdkPixbuf;
-const Clutter = imports.gi.Clutter;
-const Cogl = imports.gi.Cogl;
 const Soup = imports.gi.Soup;
-
 const _ = imports.gettext.gettext;
 
+const METRIC_SYSTEM = 1;
+const IMPERIAL_SYSTEM = 2;
+
+//List of locales using imperial system according to glibc locale database
+const IMPERIAL_LOCALES = ['unm_US', 'es_US', 'es_PR', 'en_US', 'yi_US'];
+
 let debugInit = false;
 let debugEnabled = false;
 
 let _iconStore = {};
 
+let measurementSystem = null;
+
 function debug(str) {
     if (!debugInit) {
         let env = GLib.getenv('MAPS_DEBUG');
@@ -82,17 +88,82 @@ function clearGtkClutterActorBg(actor) {
                                                        alpha: 0 }));
 }
 
-function initActions(actionMap, simpleActionEntries, context) {
-    simpleActionEntries.forEach(function(entry) {
-        let action = new Gio.SimpleAction(entry.properties);
-
-        for(let signalHandler in entry.signalHandlers) {
-            let callback = entry.signalHandlers[signalHandler];
-            action.connect(signalHandler, callback.bind(context));
-        }
+function addActions(actionMap, entries) {
+    for(let name in entries) {
+        let entry = entries[name];
+        let action = createAction(name, entry);
 
         actionMap.add_action(action);
-    });
+
+        if(entry.accels)
+            setAccelsForActionMap(actionMap, name, entry.accels);
+    }
+}
+
+function setAccelsForActionMap(actionMap, actionName, accels) {
+    let app;
+    let prefix;
+
+    if(actionMap instanceof Gtk.Application) {
+        app = actionMap;
+        prefix = "app";
+    } else if(actionMap instanceof Gtk.Window) {
+        app = actionMap.application;
+        prefix = "win";
+    }
+    app.set_accels_for_action(prefix + '.' + actionName, accels);
+}
+
+function createAction(name, { state, paramType, onActivate, onChangeState }) {
+    let entry = { name: name };
+
+    if(Array.isArray(state)) {
+        let [type, value] = state;
+        entry.state = new GLib.Variant.new(type, value);
+    }
+
+    if(paramType !== undefined)
+        entry.parameter_type = GLib.VariantType.new(paramType);
+
+    let action = new Gio.SimpleAction(entry);
+
+    if(onActivate)
+        action.connect('activate', onActivate);
+    if(onChangeState)
+        action.connect('change-state', onChangeState);
+
+    return action;
+}
+
+function _getPlatformData(appId, timestamp) {
+    let context = Gdk.Display.get_default().get_app_launch_context();
+    context.set_timestamp(timestamp);
+    let info = Gio.DesktopAppInfo.new(appId + '.desktop');
+    let id = new GLib.Variant('s', context.get_startup_notify_id(info, []));
+
+    return { 'desktop-startup-id': id };
+}
+
+function activateAction(appId, action, parameter, timestamp) {
+    let objectPath = '/' + appId.replace(/\./g, '/');
+    let platformData = _getPlatformData(appId, timestamp);
+    let wrappedParam = parameter ? [parameter] : [];
+
+    Gio.DBus.session.call(appId,
+                          objectPath,
+                          'org.freedesktop.Application',
+                          'ActivateAction',
+                          new GLib.Variant('(sava{sv})', [action,
+                                                          wrappedParam,
+                                                          platformData]),
+                          null,
+                          Gio.DBusCallFlags.NONE, -1, null, function(c, res) {
+                              try {
+                                  c.call_finish(res);
+                              } catch(e) {
+                                  debug('ActivateApplication: ' + e);
+                              }
+                          });
 }
 
 function CreateActorFromIconName(name) {
@@ -158,6 +229,24 @@ function writeFile(filename, buffer) {
     }
 }
 
+function getMeasurementSystem() {
+    if (measurementSystem)
+        return measurementSystem;
+
+    let locale = GLib.getenv('LC_MEASUREMENT') || GLib.get_language_names()[0];
+
+    // Strip charset
+    if (locale.indexOf('.') !== -1)
+        locale = locale.substring(0, locale.indexOf('.'));
+
+    if (IMPERIAL_LOCALES.indexOf(locale) === -1)
+        measurementSystem = METRIC_SYSTEM;
+    else
+        measurementSystem = IMPERIAL_SYSTEM;
+
+    return measurementSystem;
+}
+
 function getAccuracyDescription(accuracy) {
     switch(accuracy) {
     case Geocode.LOCATION_ACCURACY_UNKNOWN:
@@ -167,15 +256,7 @@ function getAccuracyDescription(accuracy) {
         /* Translators: Accuracy of user location information */
         return _("Exact");
     default:
-        let area =  Math.PI * Math.pow(accuracy / 1000, 2);
-
-        debug(accuracy + ' => ' + area);
-        if (area >= 1)
-            area = Math.floor(area);
-        else
-            area = Math.floor(area * 10) / 10;
-
-        return _("%f km²").format(area);
+        return prettyDistance(accuracy);
     }
 }
 
@@ -220,7 +301,7 @@ function _load_http_icon(icon, loadCompleteCallback) {
     let soup_session = _get_soup_session();
 
     soup_session.queue_message(msg, function(session, msg) {
-        if (msg.status_code != Soup.KnownStatusCode.OK) {
+        if (msg.status_code !== Soup.KnownStatusCode.OK) {
             log("Failed to load pixbuf: " + msg.reason_phrase);
             return;
         }
@@ -279,13 +360,26 @@ function prettyTime(time) {
 }
 
 function prettyDistance(distance) {
-    let m = Math.floor(distance);
-    let km = Math.floor(m/1000);
-    m = m % 1000;
+    distance = Math.round(distance);
 
-    if (km > 0)
-        return _("%f km").format(km);
-    else if (m > 0)
-        return _("%f m").format(m);
-    return '';
+    if (getMeasurementSystem() === METRIC_SYSTEM){
+        if (distance >= 1000) {
+            distance = Math.round(distance / 1000 * 10) / 10;
+            /* Translators: This is a distance measured in kilometers */
+            return _("%f km").format(distance);
+        } else
+            /* Translators: This is a distance measured in meters */
+            return _("%f m").format(distance);
+    } else {
+        // Convert to feet
+        distance = Math.round(distance * 3.2808399);
+        if (distance >= 1056) {
+            // Convert to miles when distance is more than 0.2 mi
+            distance = Math.round(distance / 5280 * 10) / 10;
+            /* Translators: This is a distance measured in miles */
+            return _("%f mi").format(distance);
+        } else
+            /* Translators: This is a distance measured in feet */
+            return _("%f ft").format(distance);
+    }
 }
